#include "DeerStudio/EditorEngine/API/Entity.h"
#include "Deer/Enviroment.h"
#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/DockPanelObject.h"
#include "Deer/Scene.h"

#define GET_ENTITY(id) Scene::environment.getEntity(id)    

namespace Deer {
    namespace EditorEngine {
        EntityStruct activeEntity;

        EntityStruct getRoot() {
            return EntityStruct(0);
        }
        
        void constructEntityStruct(int id, void* memory) {
            new (memory)EntityStruct(id);
        }

        int EntityStruct::getId() {
            return entityId;
        }

        std::string EntityStruct::getName() {
            return GET_ENTITY(entityId)
                .getComponent<TagComponent>()
                .tag;
        }


        void EntityStruct::setName(std::string& name) {
            GET_ENTITY(entityId)
                .getComponent<TagComponent>()
                .tag = name;
        }

        void EntityStruct::destroy() {
            GET_ENTITY(entityId)
                .destroy();
        }

        bool EntityStruct::isRoot() {
            return entityId == 0;
        }

        bool EntityStruct::exists() {
            return Scene::environment
                .entityExists(entityId);
        }

        void EntityStruct::setParent(EntityStruct parent_struct) {
            Entity& parent = GET_ENTITY(parent_struct.entityId);
            
            GET_ENTITY(entityId)
                .setParent(parent);
        }

        EntityStruct EntityStruct::getParent() {
            Entity& self = GET_ENTITY(entityId);
            if (self.isRoot())
                return *this;
            
            return EntityStruct(self.getParentId());
        }

        EntityStruct getParent();

        bool EntityStruct::isDescendantOf(EntityStruct parent_struct) {
            Entity& parent = GET_ENTITY(parent_struct.entityId);
            
            return GET_ENTITY(entityId)
                .isDescendantOf(parent);
        }

        bool EntityStruct::opEquals(const EntityStruct& other) {
            return entityId == other.entityId;
        }

        EntityStruct EntityStruct::getSelf() {
            return *this;
        }

        glm::vec3 TransformComponentStruct::getPosition() {
            return GET_ENTITY(entityId).getComponent<TransformComponent>().position;
        }

        glm::vec3 TransformComponentStruct::getScale() {
            return GET_ENTITY(entityId).getComponent<TransformComponent>().scale;
        }

        glm::vec3 TransformComponentStruct::getRotation() {
            return GET_ENTITY(entityId).getComponent<TransformComponent>().getEulerAngles();
        }

        void TransformComponentStruct::setPosition(glm::vec3 value) {
            GET_ENTITY(entityId).getComponent<TransformComponent>().position = value;
        }

        void TransformComponentStruct::setScale(glm::vec3 value) {
            GET_ENTITY(entityId).getComponent<TransformComponent>().scale = value;
        }

        void TransformComponentStruct::setRotation(glm::vec3 value) {
            GET_ENTITY(entityId).getComponent<TransformComponent>().setEulerAngles(value);
        }

        int EntityChildArrayStruct::getChildCount() {
            return GET_ENTITY(entityId)
                .getComponent<RelationshipComponent>()
                .childCount;
        }

        EntityStruct EntityChildArrayStruct::getChild(int i) {
            RelationshipComponent& rc = GET_ENTITY(entityId)
                .getComponent<RelationshipComponent>();

            if (i < 0 || i >= rc.childCount) {
                DEER_UI_ENGINE_ERROR("Error while executing EntityChild.getChild(..), id {0} is invalid for child count of {1}", i, rc.childCount);
                if (currentDockPanelExecution)
                    currentDockPanelExecution->invalidate();

                return EntityStruct(0);
            }

            return EntityStruct(rc.getChildrenId(i));
        }

        EntityStruct EntityStruct::createChild(std::string& name) {
            Entity& me = GET_ENTITY(entityId);

            Entity& newEnt = Scene::environment
                .createEntity(name);

            newEnt.setParent(me);

            return EntityStruct(newEnt.getId());
        }

    }
}