#include "DeerStudio/EditorEngine/API.h"
#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/ErrorHandle.h"

#include "angelscript.h"

namespace Deer {
    void EditorEngine::registerEditorEngineFunctions() {
        registerEntityFunctions();
        registerMathFunctions();

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void textColor(float, float, float, const string& in)",
            asFUNCTION(textColor),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void text(const string& in)",
            asFUNCTION(text),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void textCenter(const string& in)",
            asFUNCTION(textCenter),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void textEnd(const string& in)",
            asFUNCTION(textEnd),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void drawIcon(const string& in, int)",
            asFUNCTION(drawIcon),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool isItemClicked(int)",
            asFUNCTION(
                isItemClicked
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool isMouseDoubleClicked(int)",
            asFUNCTION(
                isMouseDoubleClicked
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void setupAutomaticColumns(int)",
            asFUNCTION(
                setupAutomaticColumns
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void setupColumns(int)",
            asFUNCTION(
                setupColumns
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void endColumns()",
            asFUNCTION(endColumns),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void nextColumn()",
            asFUNCTION(nextColumn),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void print(const string& in)",
            asFUNCTION (print),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void drawIconCentered(const string& in, int)",
            asFUNCTION(drawIconCentered),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getResourceCount(ResourceType, const string& in)",
            asFUNCTION(getResourceCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourceNameById(ResourceType, const string& in, int)",
            asFUNCTION(getResourceNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourcePathById(ResourceType, const string& in, int)",
            asFUNCTION(getResourcePathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getDirCount(ResourceType, const string& in)",
            asFUNCTION(getDirCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirPathById(ResourceType, const string& in, int)",
            asFUNCTION(getDirPathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirNameById(ResourceType, const string& in, int)",
            asFUNCTION(getDirNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void treeNodeLeaf(const string& in, bool)",
            asFUNCTION(treeNode),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool treeNode(const string& in, bool, any@+, ReciverFunc@+)",
            asFUNCTION(treeNodeRecursive),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool componentNode(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(componentNode),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void space()",
            asFUNCTION(space),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void contextItemPopup(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(contextItemPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void contextMenuPopup(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(contextMenuPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void modalPopup(const string& in, ReciverFunc@+)",
            asFUNCTION(modalPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void simplePopup(const string& in, ReciverFunc@+)",
            asFUNCTION(simplePopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void openPopup(const string& in, any@)",
            asFUNCTION(openPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void closePopup()",
            asFUNCTION(closePopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool menuItem(const string& in)",
            asFUNCTION(menuItem),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void dragDropSource(const string& in, any@, const string& in)",
            asFUNCTION(dragDropSource),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void dragDropTarget(const string& in, any@+, TransferFunc@+)",
            asFUNCTION(dragDropTarget),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool button(const string& in)",
            asFUNCTION(button),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool buttonCenter(const string&in)",
            asFUNCTION(buttonCenter),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool buttonEnd(const string&in)",
            asFUNCTION(buttonEnd),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool inputText(const string& in, const string& in, string& out)",
            asFUNCTION(inputText),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "float magicSlider(const string& in, float, float)",
            asFUNCTION(magicSlider),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "vec3 magicSlider3(const string& in, vec3, float)",
            asFUNCTION(magicSlider3),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void sameLine()",
            asFUNCTION(sameLine),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void separator()",
            asFUNCTION(separator),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void title(const string&in)",
            asFUNCTION(title),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void titleEnd(const string&in)",
            asFUNCTION(titleEnd),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void titleCenter(const string&in)",
            asFUNCTION(titleCenter),
            asCALL_CDECL
        ));

    }
}